package rational;

//Represents one rational number with a numerator and denominator

public class Rational
{
	private int numerator,denominator;
	
	/*Sets up the rational number by ensuring a non-zero denominator and 
	making only the numerator signed*/
	
	public Rational(int numer,int denom)
	{
		if(denom==0)
			denom=1;
		//Make the numerator store the sign
		if(denom<0)
		{
			numer*=-1;
			denom*=-1;
		}
		numerator=numer;
		denominator=denom;
		
		reduce();
	}
	
	/*Reduces this rational number by dividing both the numerator and denominator
	by their greatest common divisor.*/
	
	private void reduce()
	{
		int common;
		common=gcd(Math.abs(numerator),denominator);
		numerator/=common;
		denominator/=common;
	}
	
	/*Computes and returns the greatest common divisor of the two positive 
	parameters.Uses Euclid's algorithm.*/
	
	private int gcd(int n1,int n2)
	{
		while(n1!=n2)
			if(n1>n2)
				n1-=n2;
			else
				n2-=n1;
		return n1;
	}
	
	/*Adds this rational number to the one passed as a parameter.A common
	denominator is found by multiplying the individual denominators*/
	
	public Rational add(Rational r2)
	{
		int commonDenominator=denominator*r2.getDenominator();
		int numerator1=numerator*r2.getDenominator();
		int numerator2=r2.getNumerator()*denominator;
		int sum=numerator1+numerator2;
		
		return new Rational(sum,commonDenominator);
	}
	
	//Subtarcts the rational number passed as a parameter from this rational number.
	
	public Rational subtract(Rational r2)
	{
		int commonDenominator=denominator*r2.getDenominator();
		int numerator1=numerator*r2.getDenominator();
		int numerator2=r2.getNumerator()*denominator;
		int difference=numerator1-numerator2;
		
		return new Rational(difference,commonDenominator);
	}
	
	//Multiplies this rational number with the one passed as parameter.
	
	public Rational multiply(Rational r2)
	{
		int numer=numerator*r2.getNumerator();
		int denom=denominator*r2.getDenominator();
		
		return new Rational(numer,denom);
	}
	
	/*Divides this rational number by the one passed as parameter
	by multiplying by the reciprocal of the second rational*/
	
	public Rational divide(Rational r2)
	{
		return multiply(r2.reciprocal());
	}
	
	//Returns the reciprocal of this number
	
	public Rational reciprocal()
	{
		return new Rational(denominator,numerator);
	}
	
	//Returns the numerator of this rational number
	
	public int getNumerator()
	{
		return numerator;
	}
	
	//Returns the denominator of this rational number
	
	public int getDenominator()
	{
		return denominator;
	}
	
	/*Determines if this rational number is equal to the one passed
	as parameter.Assumes they are both reduced.*/
	
	public boolean equals(Rational r2)
	{
		return (numerator==r2.getNumerator() && denominator==r2.getDenominator());
	}
	
	//Returns the rational number as a string.
	
	public String toString()
	{
		String rational="";
		if(numerator==0)
			rational="0";
		else if(denominator==1)
			rational+=numerator;
		else
			rational=numerator+" / "+denominator;
		return rational;
	}
	
	//Following two methods do the job of gcd(int n1,int n2) and reduce() methods
	
	private int findGCD()
	{
		int smaller=(numerator<denominator)? numerator:denominator;
		int counter=smaller;
		while(numerator%counter!=0 || denominator%counter!=0)
			counter--;
		return counter;
	}
	
	private void commonize()
	{
		int count=findGCD();
		numerator/=count;
		denominator/=count;
	}
}
